nextflow.enable.dsl=2

process BIOFILECONVERTER_gff2fasta {

    input:
    tuple val(seqid), path(sequence), path(gff3)

    output:
    path("${seqid}_gene.fasta")
    path("${seqid}_ncrna.fasta")
    path("${seqid}_mrna.fasta")
    path("${seqid}_cds.fasta")
    path("${seqid}_prot.fasta")
    
    script:
    """
    BioFileConverter.pl --input-seq-file ${sequence} --input-ann-file ${gff3} --output-ann-file ${seqid}.fasta --all-contigs --kingdom ${params.kingdom}
    sed -Ei 's/>.+ locus_tag=/>/' ${seqid}_*.fasta
    """
}

process BIOFILECONVERTER_sortGff3 {

    input:
    tuple val(seqid), path(gff3files)

    output:
    tuple val(seqid), path("${seqid}.gff3")
    
    script:
    """
    cat ${gff3files} | sort -u > tmp.infile.gff3
    BioFileConverter.pl  --all-contigs --kingdom ${params.kingdom} --input-ann-format GFF3 --output-ann-format GFF3 --input-ann-file  tmp.infile.gff3 --output-ann-file ${seqid}.gff3
    """
}


process BIOFILECONVERTER_sortGff3Final {

    input:
    tuple val(seqid), path(gff3file)

    output:
    tuple val(seqid), path("${seqid}.locus_tag.gff3")
    
    script:
    """
    BioFileConverter.pl --renumber-locus_tags ${params.locus_tag_prefix}_${seqid}g......,10,10  --all-contigs --kingdom ${params.kingdom} --input-ann-format GFF3 --output-ann-format GFF3 --input-ann-file  ${gff3file} --output-ann-file ${seqid}.locus_tag.gff3
    """
}
 
process BIOFILECONVERTER_softwareInfo {
	output:
	path("biofileconverter.software_info.tsv")

	script:
	"""
	software_info > biofileconverter.software_info.tsv
	"""
}