nextflow.enable.dsl=2

process BUSCO_protein {

	publishDir "${params.outdir}/07_busco/", mode: 'copy'
    label "process_high"

    input:
    path(proteins)

    output:
	path("proteome_mode_${params.busco_lineage_dataset}")
	
    script:
    """
    busco -f --mode prot --in ${proteins} --lineage_dataset ${params.busco_lineage_dataset} --tar --quiet -c ${task.cpus} --out_path . --out proteome_mode_${params.busco_lineage_dataset}
    """
}

process BUSCO_genome {

	publishDir "${params.outdir}/07_busco/", mode: 'copy'
    label "process_high"

    input:
    path(genome)

    output:
	path("genome_mode_${params.busco_lineage_dataset}")
	
    script:
    """
    busco -f --mode geno --in ${genome} --lineage_dataset ${params.busco_lineage_dataset} --tar --quiet -c ${task.cpus} --out_path . --out genome_mode_${params.busco_lineage_dataset}
    """
}

process BUSCO_summary {

	publishDir "${params.outdir}/07_busco/", mode: 'copy'
	errorStrategy { task.exitStatus in 0..1 ? 'ignore' : 'terminate' }
	
	input:
	path(busco_protein_dir)
	path(busco_genome_dir)
	
	output:
	path("summary.busco.${params.busco_lineage_dataset}.tsv")
	path("summary.busco.${params.busco_lineage_dataset}.bed")

	script:
	"""
	echo "# Missing in annotation and found in genomic busco" >  summary.busco.${params.busco_lineage_dataset}.tsv
	cat ${busco_protein_dir}/run_${params.busco_lineage_dataset}/missing_busco_list.tsv  | xargs -i grep -w {} ${busco_genome_dir}/run_${params.busco_lineage_dataset}/full_table.tsv  | grep -v Missing | sed 's/#/##/' >> summary.busco.${params.busco_lineage_dataset}.tsv
	echo "# Fragmented in annotation and Complete in genomic busco" >>  summary.busco.${params.busco_lineage_dataset}.tsv
	grep '#\\|Fragmented'  ${busco_protein_dir}/run_${params.busco_lineage_dataset}/full_table.tsv | cut -f1 | xargs -i grep -w {} ${busco_genome_dir}/run_${params.busco_lineage_dataset}/full_table.tsv | grep '#\\|Complete' | sed 's/#/##/' >>  summary.busco.${params.busco_lineage_dataset}.tsv
	grep -v ^# summary.busco.${params.busco_lineage_dataset}.tsv | cut -f 1,2 | sed 's/\\t/_/g' > tmp.status
	grep -v ^# summary.busco.${params.busco_lineage_dataset}.tsv | cut -f 3,4,5 > tmp.pos
	grep -v ^# summary.busco.${params.busco_lineage_dataset}.tsv | cut -f 7 > tmp.score
	grep -v ^# summary.busco.${params.busco_lineage_dataset}.tsv | cut -f 6 > tmp.strand
	paste tmp.pos tmp.status tmp.score tmp.strand | sort -k1,1 -k2n,2 > summary.busco.${params.busco_lineage_dataset}.bed
	echo "# Missing in genomic busco and Found in annotation" >>  summary.busco.${params.busco_lineage_dataset}.tsv
	grep '#\\|Missing' ${busco_genome_dir}/run_${params.busco_lineage_dataset}/full_table.tsv | cut -f1 | xargs -i grep -w {} ${busco_protein_dir}/run_${params.busco_lineage_dataset}/full_table.tsv | grep -v Missing | sed 's/#/##/'>> summary.busco.${params.busco_lineage_dataset}.tsv
	[ -s summary.busco.${params.busco_lineage_dataset}.bed ] && touch summary.busco.${params.busco_lineage_dataset}.bed || echo "#no_differences" > summary.busco.${params.busco_lineage_dataset}.bed
	"""
	
}

process BUSCO_softwareInfo {
	output:
	path("busco.software_info.tsv")

	script:
	"""
	software_info > busco.software_info.tsv
	"""
}