nextflow.enable.dsl=2

process EUGENE_trainIMM {
    publishDir "${params.outdir}/02_egn_param", mode: 'copy'

    input:
    path(cdsfile)
    path(intronfile)

    output:
    path("eugene.mat")
 
    script:
    """
    egn_TrainIMM ${cdsfile} ${intronfile} && cp ${cdsfile}.bin eugene.mat
    """
}

process EUGENE_run {
    tag ("${seqid}")

	input:
	path (cfg)
    path (matrix)
	tuple val (seqid), path(sequence), path(evidenceFiles)
    val (prefix)
    
	output:
	tuple val (seqid), path ("./${prefix}${seqid}.gff3")

	

	script:
	extension = ''
	if (prefix == 'ANNOT.RV')
	{
		extension = 'RV'
	}
	else if (prefix == 'ANNOT.FW')
	{
		extension = 'FW'
	}
	if (extension != '')
	{
	"""
	ls ${seqid}.blast[0123456789].${extension}.gff3 | sed 's/.${extension}.gff3//' | xargs -i echo "rm {}.gff3 && ln -s {}.${extension}.gff3 {}.gff3" > relink.bash
	bash relink.bash
	eugene -A ${cfg} -m ${matrix} ${params.eugene_params} -pg -O  ./${prefix} ${sequence} > ${prefix}${seqid}.eugene.stdout 2> ${prefix}${seqid}.eugene.stderr
	"""
	}
	else 
	{
	"""
	eugene -A ${cfg} -m ${matrix}  ${params.eugene_params} -pg -O  ./${prefix} ${sequence} > ${prefix}${seqid}.eugene.stdout 2> ${prefix}${seqid}.eugene.stderr
	"""
	}
	
}

process EUGENE_runBigGenome {
    tag ("${seqid}")

    label "process_high_memory"
	input:
	path (cfg)
    path (matrix)
	tuple val (seqid), path(sequence), path(evidenceFiles)
    val (prefix)
    
	output:
	tuple val (seqid), path ("./${prefix}${seqid}.gff3")

	script:
	extension = ''
	if (prefix == 'ANNOT.RV')
	{
		extension = 'RV'
	}
	else if (prefix == 'ANNOT.FW')
	{
		extension = 'FW'
	}
	if (extension != '')
	{
	"""
	ls ${seqid}.blast[0123456789].${extension}.gff3 | sed 's/.${extension}.gff3//' | xargs -i echo "rm {}.gff3 && ln -s {}.${extension}.gff3 {}.gff3" > relink.bash
	bash relink.bash
	eugene -A ${cfg} -m ${matrix} -pg -O  ./${prefix} ${sequence} > ${prefix}${seqid}.eugene.stdout 2> ${prefix}${seqid}.eugene.stderr
	"""
	}
	else 
	{
	"""
	eugene -A ${cfg} -m ${matrix} -pg -O  ./${prefix} ${sequence} > ${prefix}${seqid}.eugene.stdout 2> ${prefix}${seqid}.eugene.stderr
	"""
	}
	
}

process EUGENE_softwareInfo {
	output:
	path("eugene.software_info.tsv")

	script:
	"""
	software_info > eugene.software_info.tsv
	"""
}