nextflow.enable.dsl=2

include { GMAP_updConf;GMAP_reportSummary;GMAP_map;GMAP_concat; GMAP_concat as GMAP_concatRmSpliced;GMAP_postProcessing } from '../software/gmap/main.nf'
include { LIPMUTILS_trimPolyX;LIPMUTILS_splitEstDBs} from '../software/lipmutils/main.nf'
include { EGNPCORE_splitGff3BySeqid; EGNPCORE_splitGff3BySeqid as EGNPCORE_splitGff3BySeqidRmSpliced;EGNPCORE_justPublish }  from '../software/egnpcore/main.nf'
include {EGNPCORE_PublishEvidences;EGNPCORE_dumpEstCfg }  from '../software/egnpcore/main.nf'

workflow gmap {

	take:
		configMap
		genomeMode
		genomeHash
		completeGenome
		gmapIdx

	main:
		
		estDBsCh = Channel.from(configMap.est)
		

		(estDBsCh,estNewConf) = LIPMUTILS_trimPolyX(estDBsCh)

		estDBsCh = estDBsCh.map { t ->
			def estName = t.get(0)
			def estFile = t.get(1)
			def conf = configMap.est.find {element -> element.name == estName}
			conf.file = estFile
			conf
		}
		

		estCfgUpd0File = EGNPCORE_dumpEstCfg(estNewConf.collect())
		
		splitEstDBsCh = LIPMUTILS_splitEstDBs(estDBsCh).flatten().map{ 
			n -> {
					splitfile = n
					basename = splitfile.baseName
					conf = configMap.est.find {element -> element.name == basename}
					tuple(conf.name, conf, splitfile)
				}
		}

		trainingSplitEstDBch = splitEstDBsCh.filter{it.get(1).training}.map { t -> t.get(2) }
		
			


		(estGff3, estRmSplicedGff3, rawGmapGff3) = GMAP_map(completeGenome.collect(),gmapIdx.collect(), genomeMode, genomeHash, splitEstDBsCh)
		estCatGff3 = GMAP_concat(estGff3.groupTuple(), '')
		estRmSplicedCatGff3 = GMAP_concatRmSpliced(estRmSplicedGff3.groupTuple(),".rm_unspliced_${params.gmap_MIN_LEN_SHORT_UNSPLICED}")
		Utils.info("Keep raw gmap files for training db ${configMap.train_est_db.name}",params.verbose)
		rawGmapGff34training = rawGmapGff3.filter{it.get(0) == configMap.train_est_db.name}
		
		gff3FilesPath = EGNPCORE_splitGff3BySeqid(estCatGff3,completeGenome.collect(),'')
		gff3RmSplicedFilesPath = EGNPCORE_splitGff3BySeqidRmSpliced(estRmSplicedCatGff3,completeGenome.collect(),".rm_unspliced_${params.gmap_MIN_LEN_SHORT_UNSPLICED}")

		allGff3Tuples = gff3FilesPath.concat(gff3RmSplicedFilesPath).splitText().map{ n -> 
			def f = n.trim()
			def basename = f.split('/').last()
			def seqid = basename.replaceAll("\\.est\\d+","").replaceAll("\\.rm_unspliced_${params.gmap_MIN_LEN_SHORT_UNSPLICED}","").replaceAll("\\.gff3","")
			tuple(seqid, f)
		}
		
		(finalFilesTuple,reportFiles, cfgFiles) = GMAP_postProcessing(allGff3Tuples.groupTuple(),estCfgUpd0File)
		GMAP_reportSummary(reportFiles.collect())
		//TODO le first() fait que le job est toujours rejoue j'ai l'impression ... passer par un filter du premier configMap.est_list[0]  ?
		allEstConf = GMAP_updConf(cfgFiles.collect())

	
		estEvidenceFiles = finalFilesTuple.map { t ->
			t.get(1).flatten()
		}.flatten().filter{ ! it.name.contains('.RV.') && ! it.name.contains('.FW.')}.map{ f -> 
				def basename = f.name
                def dataset = basename.replaceAll("^[^\\.]+\\.","").replaceAll("\\.gff3","") 
                tuple(dataset,f) 
			}.groupTuple()
		
		EGNPCORE_PublishEvidences(estEvidenceFiles)
		
		if (params.publish_workdir)
		{
			files = EGNPCORE_justPublish(finalFilesTuple)
			ok = files.collect()
		}
		
	emit: 
	finalFilesTuple
	rawGmapGff34training
	allEstConf
	trainingSplitEstDBch
	


}
