nextflow.enable.dsl=2

include { barrnap } from './barrnap.nf'
include { rfamscan } from './rfamscan.nf'
include { red } from './red.nf'
include { blastx } from './blast.nf'
include { ltrharvest } from './ltrharvest.nf'
include { trnascanse } from './trnascanse.nf'
include { gmap } from './gmap.nf'

include { LIPMUTILS_createOutputDirectory;LIPMUTILS_fasta2overlappingwins;LIPMUTILS_fasta2tree} from '../software/lipmutils/main.nf'
include { EGNPCORE_splitGff3BySeqid;EGNPCORE_justPublish;EGNPCORE_splitByStrand } from '../software/egnpcore/main.nf'
include {EGNPCORE_PublishEvidences as EGNPCORE_PublishEvidences_trnascanse}  from '../software/egnpcore/main.nf'
include {EGNPCORE_PublishEvidences as EGNPCORE_PublishEvidences_rfamscan}  from '../software/egnpcore/main.nf'
include {EGNPCORE_PublishEvidences as EGNPCORE_PublishEvidences_barrnap}  from '../software/egnpcore/main.nf'
include {EGNPCORE_PublishEvidences as EGNPCORE_PublishEvidences_red}  from '../software/egnpcore/main.nf'
include {EGNPCORE_PublishEvidences as EGNPCORE_PublishEvidences_ltrharvest}  from '../software/egnpcore/main.nf'
include {GMAP_index}  from '../software/gmap/main.nf'

workflow seqArbo {
	take: completeGenome
	main:
		(sequencesFof, tt) = LIPMUTILS_createOutputDirectory(completeGenome)
		sequenceCh = sequencesFof.splitText()
	emit: sequenceCh
}

workflow genomeIndex {
	take: 
		completeGenome
		genomeMode
		genomeHash
		
	main:
		indexPath = GMAP_index(completeGenome,genomeMode,genomeHash)
		
	emit: indexPath
}


workflow buildOverlappingSequences {
	take: completeGenome
	main:
		windows = LIPMUTILS_fasta2overlappingwins(completeGenome, 1)
		(sequencesFof, tt) = LIPMUTILS_fasta2tree(windows)
		overlappingSequenceCh = sequencesFof.splitText()
	emit: overlappingSequenceCh
}

workflow buildLargeOverlappingSequences {
	take: completeGenome
	main:
		windows = LIPMUTILS_fasta2overlappingwins(completeGenome, params.largeWindowFactor)
		(sequencesFof, tt) = LIPMUTILS_fasta2tree(windows)
		overlappingSequenceCh = sequencesFof.splitText()
	emit: overlappingSequenceCh
}

workflow processAdditionalFiles {
	take:
		completeGenome
		configMap
	
	main:
		additionalTuples = Channel.from(configMap.additional).map { t -> 
			tuple(t.name, t.file)
		}
		
		gff3FilesTuples = EGNPCORE_splitGff3BySeqid(additionalTuples,completeGenome.collect(),'').splitText().map{ n -> 
			def f = n.trim()
			def basename = f.split('/').last()
			def seqid = basename.replaceAll("\\.\\S+\\.gff3","")
			tuple(seqid, f)
		}
		
		if (params.publish_workdir)
		{
			files = EGNPCORE_justPublish(gff3FilesTuples)
			ok = files.collect()
		}
	emit:
		gff3FilesTuples
}


workflow abInitioPredictions {
	
	take:
		completeGenome
		pseudomolecules
		overlappingSeq
		largeOverlappingSeq

	main:
		ltrharvestTuples = ltrharvest(largeOverlappingSeq)
		EGNPCORE_PublishEvidences_ltrharvest(filterEvidenceFiles(ltrharvestTuples,'ltrharvest'))
	
		maskingEvidences = ltrharvestTuples
		
		redTuples = red(completeGenome,pseudomolecules)
		
		if (! params.skip_red) {	
			EGNPCORE_PublishEvidences_red(filterEvidenceFiles(redTuples,'red'))
		}

		maskingEvidences = maskingEvidences.concat(redTuples)
		
		

		if (! params.skip_ncrna_detection )
		{
			if (! params.skip_trnascan) {	
				trnascanseTuples = trnascanse(largeOverlappingSeq) 
			}
	  		if (! params.skip_barrnap) { 
				barrnapTuples = barrnap(largeOverlappingSeq) 
	
			}
	  		if (! params.skip_rfamscan) { 
				rfamscanTuples = rfamscan(overlappingSeq)
			}
		}

	  	abInitioEvidences = maskingEvidences
		ncRnaEvidences = Channel.empty()
		if (! params.skip_ncrna_detection )
		{
			if (! params.skip_trnascan) {	
				ncRnaEvidences = ncRnaEvidences.concat(trnascanseTuples)
				EGNPCORE_PublishEvidences_trnascanse(filterEvidenceFiles(trnascanseTuples,'trnascan'))
			}
	  		if (! params.skip_barrnap) { 
				ncRnaEvidences = ncRnaEvidences.concat(barrnapTuples)
	    		EGNPCORE_PublishEvidences_barrnap(filterEvidenceFiles(barrnapTuples,'barrnap'))
			}
	  		if (! params.skip_rfamscan) { 
				ncRnaEvidences = ncRnaEvidences.concat(rfamscanTuples)
				EGNPCORE_PublishEvidences_rfamscan(filterEvidenceFiles(rfamscanTuples,'rfamscan'))
			}
		}
		
		abInitioEvidences = abInitioEvidences.concat(ncRnaEvidences)
		
	emit:
	abInitioEvidences
	maskingEvidences
	ncRnaEvidences
}

workflow estPredictions {
	take:
		configMap
		genomeMode
		genomeHash
		completeGenome
		genomeIndexPath
		
	main:
		(estEvidences,rawFilesPath,allEstConf,trainingSplitEstDBch) = gmap(configMap,genomeMode,genomeHash,completeGenome,genomeIndexPath)

	emit:
		estEvidences
		rawFilesPath
		allEstConf
		trainingSplitEstDBch

}

workflow proteinPredictions {
	take:
		configMap
		overlappingSeq
		

	main:
		proteinEvidences = blastx(configMap,overlappingSeq)
	
	emit:
		proteinEvidences
}

def filterEvidenceFiles(intuples, source)
{
	outfiles = intuples.groupTuple().map { t -> t.get(1).flatten() }.flatten().filter{ ! it.name.contains('.RV.') && ! it.name.contains('.FW.')}.map{ f -> tuple(source, f)}.groupTuple()
	return 	outfiles
}
