#!/bin/bash

ROOT_URL="http://eugene.toulouse.inra.fr/Downloads/dbs/"
LIST="plant fungi nematodes oomycetes arthropods"


if (( $# < 2 ))
then
	echo "Usage: $0 <$LIST> <local or profile name>"
	exit 1
fi

KINGDOM=$1
PROFILE=$2

[[ $LIST =~ (^|[[:space:]])$KINGDOM($|[[:space:]]) ]] && OK=1 || OK=0

if [ $OK == 0 ]
then
	echo "ERROR - kingdom $KINGDOM not in '$LIST'"
	exit 1
fi


SCRIPT_DIR="$( cd -- "$( dirname -- "${BASH_SOURCE[0]:-$0}"; )" &> /dev/null && realpath .. 2> /dev/null; )";
VERSION=`cat $SCRIPT_DIR/VERSION`


echo "[INFO] SCRIPT_DIR=${SCRIPT_DIR}"
echo "[INFO] VERSION=${VERSION}"
echo "[INFO] ROOT_URL=${ROOT_URL}"
echo "[INFO] KINGDOM=${KINGDOM}"

mkdir -p $SCRIPT_DIR/test

if [ ! -e $SCRIPT_DIR/db/uniprot_trembl_cleaned.$KINGDOM.fasta ]; then
	$SCRIPT_DIR/utils/getDbs.sh $KINGDOM
fi


if [ ! -e $SCRIPT_DIR/test/test.$KINGDOM.genome.fasta ]; then
	echo "[INFO] Retrieve $KINGDOM genome and transcripts datasets"
	cd $SCRIPT_DIR/test && wget -q $ROOT_URL/$VERSION/test.$KINGDOM.genome.fasta.gz -O - | gzip -cd > test.$KINGDOM.genome.fasta
	cd $SCRIPT_DIR/test && wget -q $ROOT_URL/$VERSION/test.$KINGDOM.transcripts.fasta.gz -O - | gzip -cd > test.$KINGDOM.transcripts.fasta
	cd $SCRIPT_DIR/test && wget -q $ROOT_URL/$VERSION/test.$KINGDOM.config.gz -O - | gzip -cd > test.$KINGDOM.config
fi



DATE=`date +%Y%m%d`
echo "[INFO] Launch pipeline using profile $PROFILE"
$SCRIPT_DIR/egn-ep.sh --fg --sequences $SCRIPT_DIR/test/test.plant.genome.fasta --config $SCRIPT_DIR/test/test.plant.config --verbose -profile debug,$PROFILE
mv $SCRIPT_DIR/runs/test.plant.genome-9596e799/latest $SCRIPT_DIR/runs/test.plant.genome-9596e799/$DATE-default

echo "[INFO] Launch pipeline using profile $PROFILE and soft masked genome only"
$SCRIPT_DIR/egn-ep.sh --fg --sequences $SCRIPT_DIR/test/test.plant.genome.fasta --soft_masked_genome --skip_repeat_masking --config $SCRIPT_DIR/test/test.plant.config --verbose -profile debug,$PROFILE
mv $SCRIPT_DIR/runs/test.plant.genome-9596e799/latest $SCRIPT_DIR/runs/test.plant.genome-9596e799/$DATE-soft_masked_genome-skip_repeat_masking

echo "[INFO] Launch pipeline using profile $PROFILE soft masked genome and internal repeat masking"
$SCRIPT_DIR/egn-ep.sh --fg --sequences $SCRIPT_DIR/test/test.plant.genome.fasta --soft_masked_genome --config $SCRIPT_DIR/test/test.plant.config --verbose -profile debug,$PROFILE
mv $SCRIPT_DIR/runs/test.plant.genome-9596e799/latest $SCRIPT_DIR/runs/test.plant.genome-9596e799/$DATE-soft_masked_genome

exit 0
