#!/bin/bash

ROOT_URL="http://eugene.toulouse.inra.fr/Downloads/dbs/"
LIST="plant fungi nematodes oomycetes arthropods"


if (( $# < 1 ))
then
	echo "Usage: $0 $LIST"
	exit 1
fi

KINGDOM=$1
[[ $LIST =~ (^|[[:space:]])$KINGDOM($|[[:space:]]) ]] && OK=1 || OK=0

if [ $OK == 0 ]
then
	echo "ERROR - kingdom $KINGDOM not in '$LIST'"
	exit 1
fi


BUSCO_DATASET=""


if [ $KINGDOM == 'plant' ]; then
     BUSCO_DATASET="viridiplantae_odb10"
elif [ $KINGDOM == 'fungi' ]; then
     BUSCO_DATASET="fungi_odb10"
elif [ $KINGDOM == 'nematodes' ]; then
     BUSCO_DATASET="nematoda_odb10"
elif [ $KINGDOM == 'oomycetes' ]; then
     BUSCO_DATASET="stramenopiles_odb10"
elif [ $KINGDOM == 'arthropods' ]; then
     BUSCO_DATASET="arthropoda_odb10"
fi

SCRIPT_DIR="$( cd -- "$( dirname -- "${BASH_SOURCE[0]:-$0}"; )" &> /dev/null && realpath .. 2> /dev/null; )";
VERSION=`cat $SCRIPT_DIR/VERSION`

echo "[INFO] SCRIPT_DIR=${SCRIPT_DIR}"
echo "[INFO] VERSION=${VERSION}"
echo "[INFO] ROOT_URL=${ROOT_URL}"
echo "[INFO] KINGDOM=${KINGDOM}"
echo "[INFO] BUSCO_DATASET=${BUSCO_DATASET}"

mkdir -p $SCRIPT_DIR/db

echo "[INFO] Retrieve $KINGDOM processed databases (TrEMBL and Reference genome proteome)"
cd $SCRIPT_DIR/db && wget -q $ROOT_URL/$VERSION/uniprot_trembl_cleaned.$KINGDOM.fasta.gz -O - | gzip -cd > uniprot_trembl_cleaned.$KINGDOM.fasta
cd $SCRIPT_DIR/db && wget -q $ROOT_URL/$VERSION/refProteome.$KINGDOM.fasta.gz -O - | gzip -cd > refProteome.$KINGDOM.fasta

echo "[INFO] Retrieve generic processed databases (Swiss-Prot, Dfam, TransposonPSI db)"
cd $SCRIPT_DIR/db && wget -q $ROOT_URL/$VERSION/uniprot_sprot_cleaned.fasta.gz  -O - | gzip -cd > uniprot_sprot_cleaned.fasta
cd $SCRIPT_DIR/db && wget -q $ROOT_URL/$VERSION/Dfam-3.6.consensus.fasta.gz -O - | gzip -cd > Dfam-3.6.consensus.fasta
cd $SCRIPT_DIR/db && wget -q $ROOT_URL/$VERSION/transposon_db.pep.gz -O - | gzip -cd > transposon_db.pep

echo "[INFO] Create default configuration file: $SCRIPT_DIR/egnep.default.$KINGDOM.cfg"
cd $SCRIPT_DIR/
sed  "s,#KINGDOM#,$KINGDOM,;s,#BUSCO_DATASET#,$BUSCO_DATASET," $SCRIPT_DIR/egnep.cfg.template > $SCRIPT_DIR/egnep.default.$KINGDOM.cfg



exit 0
