nextflow.enable.dsl=2

include { BLASTX_indexProtDB;BLASTX_cleanProtDB;BLASTX_alignHigh;BLASTX_alignMedium;BLASTX_alignLow;BLASTX_postProcessing } from '../software/blast/main.nf'
include { LIPMUTILS_m8ToGff3 }  from '../software/lipmutils/main.nf'
include {EGNPCORE_justPublish; EGNPCORE_PublishEvidences;EGNPCORE_changeBlastxCoordinatesBatch }  from '../software/egnpcore/main.nf'


workflow blastx {
	take:
		configMap
		overlappingSequenceCh
	
	main:
		proteinDBsCh = Channel.from(configMap.protein).map { conf -> tuple(conf, conf.file)}

		cleanDBs = Channel.empty()
		toCleanCh = proteinDBsCh.filter {it.get(0)['remove_repeat'] != 0}
		notToCleanCh = proteinDBsCh.filter {it.get(0)['remove_repeat'] == 0}
		cleanDBs = cleanDBs.concat(BLASTX_indexProtDB(notToCleanCh))
		cleanDBs = cleanDBs.concat(BLASTX_cleanProtDB(toCleanCh,configMap['repeat']))
		

		m8Tuples = BLASTX_alignHigh(cleanDBs.filter {it.get(1).mode == 'high' }.combine(overlappingSequenceCh))
					.concat( BLASTX_alignMedium(cleanDBs.filter{it.get(1).mode == 'medium'}.combine(overlappingSequenceCh)))
					.concat( BLASTX_alignLow(cleanDBs.filter{it.get(1).mode == 'low'}.combine(overlappingSequenceCh)))
		
		outFilesTuples = m8Tuples.map {
			t -> 
			def seqid = t.get(0).split(":").first()
			def source = t.get(1)
			def file = t.get(3)
			tuple (seqid,source, file)
		}
		

		mergedM8Tuples = EGNPCORE_changeBlastxCoordinatesBatch(outFilesTuples.groupTuple(by:[0,1]))
		.map { t -> tuple(t.get(1), t.get(0), t.get(2))} // reorder tuple to allow join (first element of left channel is the key )
		.combine(cleanDBs, by:0)
		.map{ x -> {
			def conf = x.get(3)
			conf.file = x.get(4)
			conf.seqlen = x.get(5)
			conf.index = x.get(6)
			tuple(conf, x.get(1), x.get(2)) }
			} // filter only necessary elements

		gff3Tuples = LIPMUTILS_m8ToGff3(mergedM8Tuples)
		filteredGff3Tuples = BLASTX_postProcessing(gff3Tuples)
		
		proteinEvidenceFiles = filteredGff3Tuples.map { t ->
			t.get(1).flatten()
		}.flatten().filter{ ! it.name.contains('.RV.') && ! it.name.contains('.FW.')}.map{ f -> 
				def basename = f.name
                def dataset = basename.replaceAll("^[^\\.]+\\.","").replaceAll("\\.gff3","") 
                tuple(dataset,f) 
			}.groupTuple()
		
		EGNPCORE_PublishEvidences(proteinEvidenceFiles)
		
		if (params.publish_workdir)
		{
			files = EGNPCORE_justPublish(filteredGff3Tuples)
		}

	emit:
	filteredGff3Tuples


}
